#!/bin/bash

# --------------------------------------------------------------------
# 
# Verified Systems International GmbH
# Parkstrasse 78
# D-28209 Bremen
# Germany
# Tel.  : +49 421 344576
# Fax   : +49 421 344586
# e-mail: info@verified.de
# 
# --------------------------------------------------------------------
# 
# (C) Copyright Verified Systems International GmbH $Date: 2006/11/15 11:04:34 $
# 
# -------------------------------------------------------------------
# 
# Product: RT-Tester
# 
# -------------------------------------------------------------------
# 
# File Identification: gencodecov.sh
# 
#  $Id: gencodecov.sh,v 1.1 2006/11/15 11:04:34 hartmann Exp $
# 
#  $Revision: 1.1 $
# 
#  First Edition by: Jan Peleska
# 
#  Last Change by $Author: hartmann $
# 
# -------------------------------------------------------------------
# 
# Parameter description:
#     <TLA>  [{<TLA>}]        = list of TLAs for which the
#     coverage information should be calculated and presented
#     in an HTML table. At least one TLA must be supplied.
#
# ------------------------------------------------------------------
# 
# Return values:
#     0 = no errors
#     1 = an error occurs
# 
# ------------------------------------------------------------------
#
# environment varibales:
#
# ------------------------------------------------------------------

# a usage message
function usage(){

echo "usage: $0 <TLA> [{<TLA>}]"
echo "<TLA> [{<TLA>}] = list of TLAs for which the"
echo "coverage information should be calculated and presented in an HTML"
echo "table. At least one TLA must be supplied."

};

#####################################################################

# the current working directory
MY_PWD=`pwd`

# Loop over all TLAs given as command parameters
for TLA in $*
do

# now TLA carries the name of the current test package


# when the env variable is set we try to change to this directory
if [ -n "$RTT_TESTCONTEXT" ]; then

    # when the TLA-Directory is found, we have correct parameters
    if [ -d  $RTT_TESTCONTEXT/$TLA ]; then
	cd $RTT_TESTCONTEXT/$TLA
    else
	echo
	echo $0: some parameters are wrong. The path $RTT_TESTCONTEXT/$TLA does not exist. Exit.
	usage;
	exit 1
    fi

else

    # we assume that this script is called in the TLA-directory
    # of the current test baseline
    # when the TLA is found, we have correct parameters
    if [ ! -d  ../$TLA ]; then
	echo
	echo $0: some parameters are wrong. The path ../$TLA does not exist. Exit.
	usage;
	exit 1
    fi  

fi

# the first TLA is used to place the test results

if [ $TLA == $1 ]
then

# now we are in the directory where the coverage information should be placed into sub-dir 
# ./code-coverage
TARGET_TLA_DIR=`pwd`

# create code coverage directory if it does not exist
if ! [ -d code-coverage ]
then
  echo "$0: creating directory $MY_PWD/code-coverage"
  mkdir code-coverage
fi

# cleanup old coverage files
rm -f $TARGET_TLA_DIR/code-coverage/code-cov*
rm -f $TARGET_TLA_DIR/code-coverage/*.gcov

# endif for handling first TLA
fi

# now look for the source directory, where the current TLA has its 
# coverage information. 
# If we find the source directory, it is named THESOURCEDIR.

if [ -d $SOFTWAREPATH/$TLA ]
then
  THESOURCEDIR=$SOFTWAREPATH/$TLA
  echo $THESOURCEDIR
else
  echo -e "$0: Directory $TLA not found in $SOFTWAREPATH/ .\nCheck your component.  Exit.\n"
  exit 1
fi


echo "$0: generating coverage information in directory $THESOURCEDIR"

# change into the source directory where we find the coverage data for the 
# current TLA
cd $THESOURCEDIR

# remove old coverage tmp-file
rm -f code-coverage*.tmp

# this command creates a file code-coverage.tmp  with columns
# 1.: filebasename.c
# 2.: percentage-of-total-lines-covered
# 3.: total-number-of-executable-lines
make -k coverage | \
     awk '/^File / { fname=substr($2,2,length($2)-2); } \
          /^Lines executed:/ { split($0, a, ":"); split(a[2], b, " "); \
                               print fname " " b[1] " " b[3]; }' | \
     sort > code-coverage.tmp

     # this for-loop generates a file code-coverage2.tmp  with columns
     # 1.: filebasename.c
     # 2.: filebasename.c.gcov
     # 3.: number of lines which have are reachable and have not been covered
     # For calculation of 3., we disregard all lines marked by gcov as "not covered" if such a line
     #               - is marked by a /*$$$ UNREACH[ABLE : ...] $$$*/ comment
     #               - is contained in a /*$$$ BEGIN UNREACH[ABLE : ...] ... END UNREACH[ABLE] $$$*/
     #                 block
     touch code-coverage2.tmp
     for i in `awk '{print $1}' code-coverage.tmp` 
     do
       export LINES_STILL_TO_COVER="`sed -e '/\/\*\$\$\$ *UNREACH/d' $i.gcov | \
                                     sed -e  '/\/\*\$\$\$ *BEGIN *UNREACH/,/\/\*\$\$\$ *END *UNREACH/d' | \
                                     grep -e "#####:[ ]*[0-9]\+:"  | \
                                     wc -l`"  
       echo "$i $i.gcov $LINES_STILL_TO_COVER"  >> code-coverage2.tmp
     done

     # this join generates file code-coverage3.tmp with columns
     # 1.: filebasename.c
     # 2.: percentage-of-total-lines-covered 
     # 3.: total-number-of-executable-lines
     # 4.: filebasename.c.gcov
     # 5.: number of lines which have are reachable and have not been covered
     #
     join code-coverage.tmp code-coverage2.tmp  > code-coverage3.tmp

     # the next awk script appends to file code-coverage4.tmp in the target
     # directory/code-coverage with columns
     # 1.: filebasename.c   
     # 2.: filebasename.c.gcov  
     # 3.: total-number-of-executable-lines 
     # 4.: percentage-of-total-lines-covered 
     # 5.: number of covered executable lines
     # 6.: number of reachable executable lines
     # 7.: percentage of covered reachable executable lines
     # 
     awk '{ \
	  print $1 " " $4 " " $3 " " $2 " " ($3*$2)/100  " " ((($3*$2)/100) + $5) \
	      "  " 100/(((($3*$2)/100) + $5) != 0 ? ((($3*$2)/100) + $5) : 0)*($3*$2)/100 "%"}' \
          code-coverage3.tmp >>  $TARGET_TLA_DIR/code-coverage/code-coverage4.tmp
     


# copy all gcov-files needed into the test directory
for file in $(ls *.gcov 2>/dev/null)
do
    cat $file | \
    grep -v -e "^        -:    0:" | \
    perl -e '{ 
               while (<>) { 
                   /^[ ]*(-?[0-9]+|#####|-):[ ]*[0-9]+:(.*$)/; 
                   $num = $1; $line = $2; 
                   if ($num =~ /([0-9]+|#####)/) { 
                       $num = "######" if ($num eq "#####");
                       printf "%12s    %s\n", $num, $line; 
                   } else { 
                       print "                $line\n";
                   }
               }
             }' | \
    cat > $TARGET_TLA_DIR/code-coverage/$file
done

# remove all temporary files
rm -f code-coverage*.tmp


# this is the end of the for-loop over all TLAs
done


##########################################################################
# now change into target directory to produce the final HTML file
##########################################################################
cd $TARGET_TLA_DIR/code-coverage/


     # the following command generates a file code-coverage5.tmp with one line containing
     # the columns
     # 1.: sum over executable lines of all files
     # 2.: sum over all covered lines of all files
     # 3.: percentage of covered executable lines
     # 4.: sum over reachable executable lines of all files
     # 5.: percentage of covered reachable executable lines 
     awk '{ \
	     all_lines = all_lines+$3; \
             covered_lines = covered_lines+$5; \
             all_reachable_lines =  all_reachable_lines + $6; \
	  } \
          END { print all_lines " " covered_lines " " (all_lines != 0 ? (covered_lines*100)/all_lines : 0) "% " \
	        all_reachable_lines " " (all_reachable_lines != 0 ? (covered_lines*100)/all_reachable_lines : 0) "%"; \
	  }' code-coverage4.tmp > code-coverage5.tmp

     ##############################################################################
     # Now create an html-file portion containing the coverage information 
     # for all TLAs listed in the command line.
     ##############################################################################

     # the html file header
     echo "<html>" > code-coverage.html
     echo "<body>" >> code-coverage.html
     echo >> code-coverage.html
     echo "<h1> CIDS A318 DMB Software Integration Test Code Coverage" >> code-coverage.html
     echo "(Statement Coverage) </h1>" >> code-coverage.html
     #echo "<img height=100 align=right src="../../../images/verified.gif">" >> code-coverage.html 

     # the summary line
     awk '{ printf "<b> %.0f %% of a total of %d reachable lines have been covered: </b> <p>", $5, $4 }' code-coverage5.tmp >> code-coverage.html

     # the table header
     echo "<p><p><table border=3>" >> code-coverage.html
     echo "<tr>" >> code-coverage.html
     echo "<th> File </th> <th> #Executable Lines </th> <th> % Coverage (Executable Lines) </th> \
           <th> #Reachable Lines </th> <th>  % Coverage (Reachable Lines) </th> <th>  View Coverage File </th>" >> code-coverage.html
     echo "</tr>" >> code-coverage.html

     # the table contents with the following columns:
     # 1.: filebasename.c
     # 2.: total-number-of-executable-lines 
     # 3.: percentage-of-total-lines-covered
     # 4.: number of reachable executable lines
     # 5.: percentage of covered reachable executable lines
     awk '{print "<tr>"; \
           printf "<td> %s </td> <td align=right> %d </td> <td align=right> %.0f %% </td> <td align=right> %0.f </td> <td align=right> %0.f %% </td> <td> <a href=%s.gcov> %s.gcov </a> </td>", \
                  $1, $3, $4, $6, $7, $1, $1 ; \
           print "</tr>"}' \
         code-coverage4.tmp >> code-coverage.html

     # the last row of the table containing the totals
     awk '{print "<tr>"; \
           printf "<td> <b> Total: </b> </td> <td align=right> <b> %d </b> </td> <td align=right> <b> %.0f %% \
                   </b> </td> <td align=right> <b> %.0f </b> </td> <td align=right> <b> %.0f %% </b> </td> <td>  </td>", $1, $3, $4, $5; \
           print "</tr>"}' \
         code-coverage5.tmp >> code-coverage.html

     # the table end
     echo "</table>" >> code-coverage.html
     
     # the html file footer
     echo  >> code-coverage.html
     #echo "<img width=144 align=right vspace=100 src=../../../images/A318.jpg>" >> code-coverage.html
     echo "</body>" >> code-coverage.html
     echo "</html>" >> code-coverage.html

rm -f code-coverage*.tmp

# back to the initial working directory
cd $MY_PWD

echo "$0: generation completed"
exit 0

# ------------------------------------------------------------------
# 
# Change history:
# 
# $Log: gencodecov.sh,v $
# Revision 1.1  2006/11/15 11:04:34  hartmann
# TA1-RT-Tester Sample-Project
#
# Revision 1.1.1.1  2006/09/27 19:11:29  jp
# initial
#
#  
# -------------------------------------------------------------------











