/** @file
/////////////////////////////////////////////////////////////////////
//
// Verified Systems International GmbH
// Parkstrasse 78 
// D-28209 Bremen
// Germany
// Tel.  : +49 421 344576
// Fax   : +49 421 344586
// e-mail: info@verified.de
//
//--------------------------------------------------------------------
//
// (C) Copyright Verified Systems International GmbH 
//     $Date: 2006/01/05 09:21:51 $
//
//--------------------------------------------------------------------
//
// Product: RT-Tester template for test configuration / initialisation
//
//--------------------------------------------------------------------
//
// File Identification: 
// $RCSfile: RTTcontext_global.h,v $
//
// $Header: /lion/home/repository/VVTCVS/CVS/rtt-swi/tpl/RTTcontext_global.h,v 1.15 2006/01/05 09:21:51 oli Exp $
//
// $Revision: 1.15 $
//
// First edition by: M. Oliver Mller
// Last update by    $Author: oli $
//
//--------------------------------------------------------------------
//
// Description: 
//   Default contex for global stubs
//   RTT 6.x STL Preprocessor, parsing .tpl .conf .rts files
//
// -------------------------------------------------------------------
// @TABLE OF CONTENTS:		       [TOCD: 10:48 02 Aug 2004]
//
//      [0.1] DEFINITIONS
//      [0.2] global variables
//  [1] Global Function Declarations
//      [1.1] Service Functions
//      [1.2] AM and LWP context functions
//      [1.3] AM context functions
//      [1.4] LWP context functions
//      [1.5] channel context functions
//      [1.6] vector context functions
//      [1.7] Service Functions
// -------------------------------------------------------------------*/


#ifndef _RTTCONTEXT_H
#define _RTTCONTEXT_H

#ifdef __cplusplus
extern "C" {
#endif


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

// -- INCLUDES FROM THE PROJECT CONFIGURATION FILE -------------------------
#include <stdlib.h>
#include <stdio.h>

#ifdef __cplusplus
}
#endif

// -- CPP INCLUDES FROM THE PROJECT CONFIGURATION FILES --------------------

#ifdef __cplusplus
extern "C" {
#endif


// -- RTT specific includes ------------------------------------------------
#include "rtt_test_config.h"
#include "rtt_test_version.h"
#include "rtt_test_types.h"
#include "RTTverdict.h"

#include "rttstrlib.h"
#include "rttcnllib.h"
#include "rttisetlib.h"
#include "rttxserlib.h"
#include "rttctxlib.h"
#include "rttservicelib.h"
#include "rttbooleancontractlib.h"

// -- GLOBAL USER-DEFINED PRINT FUNCTION SIGNATURES -------------------------
#include "RTT_global_user_printfuns.h"


// ///////////////////////////////////////////////
// [0.1] DEFINITIONS
// ///////////////////////////////////////////////

// ///////////////////////////////////////////////
// [0.2] global variables
// ///////////////////////////////////////////////

/** @var RTTglobal_setup_t RTTgs;
 *  A global variable containing the global setup of a test case.
 */
extern volatile RTTglobal_setup_t RTTgs;

/** @var uint32_t rtt_SUT_am_id;
 *  A global variable containing the am id of the abstract machine that is
 *  specified as the system under test (SUT) in the configuration file.
 */
//extern uint32_t rtt_SUT_am_id;

/** @var uint64_t rttStartOfTestTic;
 *  The global timetick at which the test did start.
 */
extern volatile uint64_t rttStartOfTestTic;

/** @var rttStartOfTestEpoch
 *  The global time epoch used for the time tick calculation.
 */
extern volatile time_t rttStartOfTestEpoch;

/** @var uint32_t rtt_config_num_am;
 *  A global variables storing the number of activated AMs. It is set at
 *  runtime according to the value of RTT_CONFIG_NUM_AM.
 */
extern uint32_t rtt_config_num_am;

/** @var uint32_t rtt_config_num_lwp;
 *  A global variables storing the number of LWPs of a test case. It is set at
 *  runtime according to the value of RTT_CONFIG_NUM_LWP.
 */
extern uint32_t rtt_config_num_lwp;


/** @var rtt_ti_level_t rttTiLevel;
 *  The global test integration level of a test case.
 */
extern rtt_ti_level_t rttTiLevel;


// ///////////////////////////////////////////////////////////////////
// [1] Global Function Declarations
// ///////////////////////////////////////////////////////////////////


// ///////////////////////////////////////////////
// [1.1] Service Functions
// ///////////////////////////////////////////////

/**
 * Function rttAllWarnings. 
 * Returns the sum of all warnings of all defined abstract machines.
 * Warning: this function is not thread safe.
 * @return the sum of all warnings of all defined abstract machines
 */
extern long int rttAllWarnings();

/**
 * Function rttAllFailures. 
 * Returns the sum of all failures of all defined abstract machines.
 * Warning: this function is not thread safe.
 * @return the sum of all failures of all defined abstract machines
 */
extern long int rttAllFailures();

/**
 * Function rttStandardFormat. Standard logging function for unit test AMs.
 * 
 * @param amId       internal number of the abstract machine
 * @param eventFlag  condition/event/action flag
 * @param string     text string to be logged
 */
extern void rttStandardFormat(unsigned long, unsigned long, char*);

/**
 * Function rttStandardLogAM. Standard logging function for unit test AMs:
 * Always prints scratchpad (with safe terminating zero) with standard flags
 * 
 * @param amId       internal number of the abstract machine
 */
extern void rttStandardLogAM(unsigned long amId);

/**
 * Function rttAMContext_compute_verdict.Create a test verdict according
 * to context informations.
 * 
 * @param amNumber  internal number of the abstract machine
 * @param nWarnings number of warnings (fails of  @rttCheck)
 * @param nErrors   number of errors   (fails of  @rttAssert)
 * 
 */
extern void rttAMContext_compute_verdict(unsigned long, 
					 unsigned long,
					 unsigned long);

// ///////////////////////////////////////////////
// [1.2] AM and LWP context functions
// ///////////////////////////////////////////////

/**
 * Function stublevel_context_INIT.
 * Initialisation of stub test integration levels
 */
extern void stublevel_context_INIT();

/**
 * Function stublevel_context_FINIT.
 * Finalisation of stub test integration levels
 */
extern void stublevel_context_FINIT();

// ///////////////////////////////////////////////
// [1.3] AM context functions
// ///////////////////////////////////////////////

/**
 * Function am_context_INIT.
 * Initialisation of AM context.
 */
extern void am_context_INIT();

/**
 * Function am_context_FINIT.
 * Finalisation of AM context.
 */
extern void am_context_FINIT();

// ///////////////////////////////////////////////
// [1.4] LWP context functions
// ///////////////////////////////////////////////

/**
 * Function lwp_context_INIT.
 * Initialisation of LWP context.
 */
extern void lwp_context_INIT();

/**
 * Function lwp_context_FINIT.
 * Finalisation of LWP context.
 */
extern void lwp_context_FINIT();

// ///////////////////////////////////////////////
// [1.5] channel context functions
// ///////////////////////////////////////////////

/**
 * Function channel_context_INIT.
 * Initialisation of channel context.
 */
void channel_context_INIT();

/**
 * Function channel_context_FINIT.
 * Finalisation of channel context.
 */
void channel_context_FINIT();

/**
 * Function rttGetCnlSpec. Retrieves the channel specification
 * setup for a given channel name on an LWP.
 *
 * @param cnl Name of the channel
 * @param lwp_id The number of the LWP.
 *
 * @TODO missing documentation
 * @return
 */
rttCnlSpec_t *rttGetCnlSpec(char *cnl, uint32_t lwp_id);

/**
 * Function rttPruneReadIndex. Remove duplicate read index array
 * initialisation. All ports of a channel of an abstract machine
 * should use the same read index.
 *
 * @param am_id Internal number of an abstract machine.
 */
void rttPruneReadIndex(uint32_t am_id);

/**
 * Function rttFreeReadIndex. Since read indices may be used by
 * multiple ports, the allocated memory must not be freed for each
 * port, but only once. This function frees the allocated memory once
 * and sets all referenced the rIdxPtr to NULL.
 *
 * @param am_id Internal number of an abstract machine.
 * @param rIdxPtr Read index pointer
 */
void rttFreeReadIndex(uint32_t am_id, uint32_t *rIdxPtr);


// ///////////////////////////////////////////////
// [1.6] vector context functions
// ///////////////////////////////////////////////


/**
 * Function vector_context_INIT.
 * Initialisation of vector context.
 */
void vector_context_INIT();

/**
 * Function vector_context_FINIT.
 * Finalisation of vector context.
 */
void vector_context_FINIT();




// ///////////////////////////////////////////////
// [1.7] Service Functions
// ///////////////////////////////////////////////

/**
 * Function rttStandardFormat. Standard logging function for unit test AMs.
 * 
 * @param amId       internal number of the abstract machine
 * @param eventFlag  condition/event/action flag
 * @param string     text string to be logged
 */
extern void rttStandardFormat(unsigned long, unsigned long, char*);


// -------------------------------------------------------------------------

#ifdef __cplusplus
          } // terminate extern "C" 
#endif


#endif

/*//////////////////////////////////////////////////////////////
//
// Change History:
//
// $Log: RTTcontext_global.h,v $
// Revision 1.15  2006/01/05 09:21:51  oli
// - re-organized global/local print function signature
//   inclusion (rt-tester/17)
//   Now: ./stubsrc/RTT_global_user_printfuns.h
//   and: <TLA>...../src//RTT_user_printfuns.h
//   where the latter include RTT_global_user_printfuns.h
//
// Revision 1.14  2005/10/07 12:52:11  oli
// added processing of .rtp/.conf file option
// CPP_INCLUDE ; "cpp_header.hh"
//
// Revision 1.13  2005/09/21 14:47:25  chref
// marked global variables that get changed in different threads as "volatile"
//
// Revision 1.12  2005/08/17 18:00:25  jp
// RT-Tester User Threads, first version
//
// Revision 1.11  2005/08/04 16:11:31  oli
// guarded header files with the block
//    #ifdef __cplusplus
//     extern "C" {
//     #endif
// in order to allow for linking with C++ Style name mangling.
// (changed typename -> type_name)
//
// Revision 1.10  2005/05/16 13:55:07  jp
// Define empty vector initialisation functions
//
// Revision 1.9  2004/08/23 10:46:30  oli
// added signature  rttStandardLogAM to global RTTcontext.h
//
// Revision 1.8  2004/08/04 14:53:21  marlex
// functions comments inserted/changed to coding standard
// for more information (and TODO) see in TODO_for_coding_standard.txt
//
// Revision 1.7  2004/08/02 14:43:01  marlex
// header added/changed for coding standard in *.l, *.y, *.c, *.h files
//
//
//
///////////////////////////////////////////////////////////////
*/
